@@ -25,13 +25,15 @@
 # Dublin 4, Ireland
 #
 
-import os, sys
-import math, random, numpy, string, types
+import os, sys, copy
+import math, random, string, types
 import re
+import numpy as np
 from datetime import datetime
 import ConfigParser, csv
 from itertools import izip, chain, repeat
 import Importer, Custom
+from Processing import Processor
 import Utilities
 from PEATDB.Ekin.Base import EkinProject, EkinDataset
 import PEATDB.Ekin.Fitting as Fitting
@@ -39,14 +41,18 @@ import PEATDB.Ekin.Fitting as Fitting
 class Pipeline(object):
     """This class does all the pipeline processing and configuration"""
 
-    __version__ = '1.0.0'
-    configsections = ['base','files','fitting','models','variables',
+    __version__ = '1.1.0'
+    configsections = ['base','files','fitting','models','variables','functions',
                       'excel','plotting','custom']
 
     def __init__(self, conffile=None):
 
+        homepath = os.path.join(os.path.expanduser('~'))
+        self.defaultpath = os.path.join(homepath, '.pipeline')
+        if not os.path.exists(self.defaultpath):
+            os.mkdir(self.defaultpath)
         if conffile==None:
-            confpath = os.path.join(os.path.expanduser('~'),'default.conf')
+            confpath = os.path.join(self.defaultpath, 'default.conf')
             self.createConfig(confpath)
         self.savedir = os.getcwd()
         self.filename = ''
@@ -54,29 +60,36 @@ class Pipeline(object):
         self.queue = {}
         self.results = []
         self.sep = '__'   #symbol for internal separator
+
         return
 
     def createConfig(self, conffile='default.conf', **kwargs):
         """Create a basic config file with default options and/or custom values"""
 
         c = ConfigParser.ConfigParser()
-        wdir = os.path.join(os.path.expanduser('~'),'workingdir')
-
+        wdir = os.path.join(self.defaultpath,'workingdir')
+        functionsconf = os.path.join(self.defaultpath,'functions.conf')
         defaults = {'base': [('format', 'databyrow'), ('rowstart', 0), ('colstart', 0), ('rowend', 0),
                         ('colend', 0), ('colheaderstart', 0),('rowheaderstart', 0),
                         ('rowheader', ''), ('colheader', ''),
                         ('rowrepeat', 0), ('colrepeat', 0), ('delimeter', ','),
-                        ('workingdir', wdir),  ('ignorecomments', 1),
-                        ('decimalsymbol', '.'), #('checkunicode', 0),
+                        ('workingdir', wdir), ('functionsconf',functionsconf),
+                        ('ignorecomments', 1),
+                        ('decimalsymbol', '.'), ('preprocess',''),
                         ('xformat',''),('yformat','')],
                     'files': [('groupbyname', 0), ('parsenamesindex', 0),
                                 ('replicates',0)],
                     'models': [('model1', '')], 'variables': [('variable1', '')],
+                    'functions':[('function1','')],
                     'excel': [('sheet', 0), ('numsheets', 1)],
-                    'plotting': [('saveplots', 0), ('normaliseplots', 0), ('grayscale', 0),
-                        ('dpi', 100)],
+                    'plotting': [('saveplots', 0), ('fontsize',9),                                    
+                            ('normalise', 0), ('grayscale', 0), ('alpha',0.8),
+                            ('font','sans-serif'), ('markersize',25), ('linewidth',1),
+                            ('showerrorbars',0), ('dpi', 100), ('marker','o'),
+                            ('markers','-,o'),('legend',0)],
                     'custom': [],
-                    'fitting': [('xerror', 0), ('yerror', 0), ('iterations', 50), ('modelsfile','')],
+                    'fitting': [('xerror', 0), ('yerror', 0), ('iterations', 50),
+                                ('modelsfile','')],
                     }
 
         cp = Utilities.createConfigParserfromDict(defaults, self.configsections ,**kwargs)
@@ -105,6 +118,7 @@ class Pipeline(object):
         #both the Importer and Pipeline object get copies of the config options
         #as attributes, convenient but probably bad..
         Utilities.setAttributesfromConfigParser(self, cp)
+
         self.models = sorted(self.models)
         self.variables = sorted(self.variables)
         print 'parsed config file ok, format is %s' %format
@@ -250,8 +264,9 @@ class Pipeline(object):
         print 'importer returned %s datasets' %l
         return
 
-    def preProcess(self):
+    def prepareData(self):
         """Prepare for data import and processing"""
+
         if self.workingdir == '':
             self.workingdir = os.path.join(os.getcwd(), 'workingdir')
         if not os.path.exists(self.workingdir):
@@ -261,12 +276,32 @@ class Pipeline(object):
             Utilities.clearDirectory(self.workingdir)
         return
 
+    def doProcessingStep(self, data, fname):
+        """Apply a pre-defined processing step to the data"""
+       
+        X = Processor(self.functionsconf)
+        names = [i[1] for i in self.functions]
+        for n in names:
+            if n not in X.predefined:
+                print 'function %s not found' %n
+                return data
+        #user may want to see raw data before processing, so we plot here
+        if self.saveplots == 1:
+            Er = self.getEkinProject(data)            
+        data = X.doFunctions(names, data)
+        if self.saveplots == 1:
+            Em = self.getEkinProject(data)      
+            Er.addProject(Em,label='proc')
+            Er.saveProject(fname+'_processed')
+            self.saveEkinPlotstoImages(Er, fname+'_processed',groupbylabel=True)
+        return data
+
     def run(self, callback=None):
         """Do initial import/fitting run with the current config"""
 
         self.stop=False
         self.loadModels()
-        self.preProcess()
+        self.prepareData()
         print 'processing files in queue..'
 
         if self.groupbyname == 1:
@@ -302,6 +337,10 @@ class Pipeline(object):
             fname = os.path.basename(key)
             fname = os.path.join(self.workingdir, fname)
             data = imported[key]
+
+            if self.function1 != '':
+                data = self.doProcessingStep(data, fname)
+
             if labels.has_key(key):
                 label = labels[key]
             else:
@@ -324,6 +363,7 @@ class Pipeline(object):
             else:
                  #if no fitting we just put the data in ekin
                 Em = self.getEkinProject(data)
+                results = data
             Em.saveProject(fname)
             Em.exportDatasets(fname)
             if self.model1 != '':
@@ -350,6 +390,7 @@ class Pipeline(object):
             self.saveEkinPlotstoImages(Em, fname)
         print 'processing done'
         print 'results saved to %s' %self.workingdir
+        self.results = results
         return results
 
     def processFits(self, rawdata, models=None, variables=None, ind=None,
@@ -442,7 +483,7 @@ class Pipeline(object):
         for n in names:
             arrs = []
             for D in dictslist:
-                arrs.append(numpy.array(D[n]))
+                arrs.append(np.array(D[n]))
             newdata[n] = list(sum(arrs)/len(arrs))
         return newdata
 
@@ -547,18 +588,56 @@ class Pipeline(object):
                 self.queue[f] = f
         return
 
-    def saveEkinPlotstoImages(self, E, filename):
-        """Save the ekin plots to png images"""
+    def getPlotOptions(self):
+        """Get options dict for passing to plotdatasets"""
+        opts = dict(self.plotting)
+        #print self.plotting
+        for k in opts:
+            if k=='markers':
+                opts[k]=list(opts[k].split(','))                
+            else:
+                try:
+                    opts[k] = eval(opts[k])
+                except:
+                    pass
+        return opts
+
+    def saveEkinPlotstoImages(self, E, filename, groupbylabel=False):
+        """Save the ekin plots to png images
+           groupbylabel allows commonly labelled datasets 
+           to be overlayed - useful for viewing processing results"""
         title = os.path.basename(filename)
         filename = os.path.join(self.workingdir, filename)
         filename = filename + '.png'
-        #limit to 25 plots per image
-        d = E.datasets
-        if len(d) > 25: d = E.datasets[:25]
-        E.plotDatasets(d, filename=filename, plotoption=2,
-                       dpi=self.dpi, size=(10,8), showerrorbars=True,
-                       title=title, normalise=self.normaliseplots,
-                       grayscale=self.grayscale)
+        #limit to 30 plots per image
+        dsets = E.datasets
+        if len(dsets) > 30: 
+            dsets = E.datasets[:30]
+        plotopts = self.getPlotOptions()
+        #the following functionality should be integrated into ekin plotter    
+        if groupbylabel == True:
+            import itertools
+            import pylab as plt
+            from mpl_toolkits.axes_grid1 import AxesGrid
+            groups = []
+            #use iterator to get groups
+            iterator = itertools.groupby(sorted(dsets), key=lambda x: x.split('_')[0])
+            for i,j in iterator:
+                groups.append(list(j))
+            #print groups
+            fig = plt.figure()
+            cols, dim = E.getGeometry(groups, 0)
+            c=1
+            for g in groups:
+                ax=fig.add_subplot(cols,dim,c)                            
+                E.plotDatasets(g,figure=fig,axis=ax,plotoption=3,
+                                varyshapes=1,**plotopts)
+                c+=1
+            #fig.tight_layout(pad=0.8, w_pad=0.8, h_pad=1.0)
+            fig.savefig(filename,dpi=plotopts['dpi'])
+        else:
+            E.plotDatasets(dsets, filename=filename, plotoption=2,
+                           size=(10,8), title=title, **plotopts)
         return
 
     def saveFitstoCSV(self, E, filename):
@@ -647,3 +726,4 @@ class FolderStructure(object):
         print '%s files found in %s folders' %(self.p)
         return
 
+
